/***************************************************************************
 *   Copyright (C) 2006 by André Lindhjem <belgarat@sdf.lonestar.org>,     *
 *                         Kjetil Holien <kjetil.holien@gmail.com>,        *
 *                         Terje Risa <terje.risa@gmail.com> &             *
 *                         Øyvind Nerbråten <oyvind@nerbraten.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *
 * \file main.c
 * \author André Lindhjem, Kjetil Holien, Terje Risa & Øyvind Nerbråten
 * \date 21.03.2006
 * 
 * A simple console front-end to demonstrate the use of libdaisy.
 */
 

#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <ncursesw/ncurses.h> /* add -lncursesw to build! */
#include <libdaisy.h>

/* callback function declarations */
void cb_audio_done (void *);
void cb_audio_next (void *);
void cb_text (void *);
void cb_id (void *);
void cb_error (enum daisy_status, char *);
void cb_progress (long int);

/* Listens for CTRL-C and does it's best to free resources in a Good Way */
static void sigproc (int);		


int main (int argc, char *argv[])
{	
	int row, col;
	int keypress = 0;	/* user input */
	
	/* creating a daisyplayer */
	daisyplayer_t daisy;
	
	argc = argc; argv = argv;
	
	/* adding signal handling. this should catch ctrl-c and free the resources because this is a Good Thing */
	(void) signal(SIGINT, sigproc);
	
	/* initializing daisyplayer with the callback functions */
	daisy = daisy_init(cb_audio_done,
			   cb_audio_next,
			   cb_text,
			   cb_id,
			   cb_error,
			   cb_progress);
			   
	
	initscr ();	/* start the curses mode */
	noecho ();	/* don't echo to screen */
	keypad (stdscr, TRUE);		/* enable keypad keys */
	scrollok(stdscr, TRUE);		/* make the window scroll automatically */
	idlok(stdscr, TRUE);
	
	row = 1, col = 10;
	mvprintw (row, col, "Welcome to:");
	row +=2, col = 1;
	mvprintw (row, col, "________         .__               __________.__                             ");
	row++;
	mvprintw (row, col, "\\______ \\ _____  |__| _________.__.\\______   \\  | _____  ___.__. ___________ ");
	row++;
	mvprintw (row, col, " |    |  \\\\__  \\ |  |/  ___<   |  | |     ___/  | \\__  \\<   |  |/ __ \\_  __ \\");
	row++;
	mvprintw (row, col, " |    `   \\/ __ \\|  |\\___ \\ \\___  | |    |   |  |__/ __ \\\\___  \\  ___/|  | \\/");
	row++;
	mvprintw (row, col, "/_______  (____  /__/____  >/ ____| |____|   |____(____  / ____|\\___  >__|   ");
	row++;
	mvprintw (row, col, "        \\/     \\/        \\/ \\/                         \\/\\/         \\/       ");
	row += 2, col = 5;
	mvprintw (row, col, "Commands:\n");
	col += 5;
	mvprintw (++row, col, "o = open diasy book\n");
	mvprintw (++row, col, "p = play\n");
	mvprintw (++row, col, "s = stop\n");
	mvprintw (++row, col, "space = pause/play\n");
	mvprintw (++row, col, "left arrow = prev passage\n");
	mvprintw (++row, col, "right arrow = next passage\n");
	mvprintw (++row, col, "up arrow = prev chapter\n");
	mvprintw (++row, col, "down arrow = next chapter\n");
	mvprintw (++row, col, "q = quit\n\n");
		
	while (keypress != 'q')		/* q quits */
	{
		char string[1000];
		keypress = wgetch (stdscr); /* get user input */

		switch (keypress)
		{
			case KEY_UP:
				daisy_seek (daisy, DAISY_SEEK_PREV_CHAPTER); /* seek */
				break;
			case KEY_DOWN:
				daisy_seek (daisy, DAISY_SEEK_NEXT_CHAPTER); /* seek */
				break;
			case KEY_LEFT:
				daisy_seek (daisy, DAISY_SEEK_PREV_PASSAGE); /* seek */
				break;
			case KEY_RIGHT:
				daisy_seek (daisy, DAISY_SEEK_NEXT_PASSAGE); /* seek */
				break;
			case ' ':
				daisy_pause (daisy); /* pause/restart playback */
				break;
			case 'p':
				printw ("\n\n");
				daisy_play (daisy); /* start playback */
				break;
			case 's':
				if (daisy_stop (daisy) == 1) {
					/* we have a successful stop */
				} else {
					/* error stopping */
				}
				break;
			case 'o':
				echo ();
				printw ("\nPlease enter full path to the Daisy book (../ncc.html | ../*.ncx)\n");
				(void) scanw("%s", string);
				noecho ();
				if (string != NULL)
					daisy_load (daisy, string); /* load supplied daisy dtb */
				printw ("\n");
				refresh (); /* refresh screen */
				break;				
			default:
				printw ("\nunknown key: %d\n", keypress);
				break;
		}
	}
	printw ("\n\nEnd of book\n");
	endwin ();
	
	/* terminates and cleans up after daisyplayer */
	daisy_term (daisy);

	return (1);
}


/**
 * The callback functon called everytime the daisy engine is done playing a audio segment.
 * @param notused - not used.
 */
void cb_audio_done (void *notused)
{
	notused = notused;
}


/**
 * This callback function is called everytime the daisy engine starts to play a new audio segment.
 * @param length - a unsigned long int containing the length of the audio segment in ms.
 */
void cb_audio_next (void *length)
{
	length = length;
}


/**
 * This callback function is called everytime the daisy engine starts to play a new audio segment, supplying the corresponding text.
 * @param passage - an char* containing the text passage.
 */
void cb_text (void *passage)
{
	char *_passage = passage;
	printw ("%s\n", _passage);
	refresh (); /* refreshing the screen */
}


/**
 * This callback function is called everytime the daisy enfine start to play a new audio segment, supplying the corresponding id to the text in the xml file.
 * @param id - an char* containing the id of the text passage in the xml file.
 */
void cb_id (void *id)
{
	char *_id = id;
	printw ("(%s) - ", _id);	/* prints ID. This line has no newline and will be displayed together with the text passage */
}


/**
 * This callback function is called everytime an error occurs in the daisy engine.
 * @param err - an enum daisy_status containing the error code of error type.
 * @param daisy_error_msg - an char* containing the error message. This can be NULL when an message is not needed to explain the error.
 */
void cb_error (enum daisy_status err, char *daisy_error_msg)
{
	printw ("main.c got error! (%d) - (%s)\n", err, daisy_error_msg);
	refresh ();
}


/**
 * This callback function is called during playback, supplying the progress in ms.
 * @param progress_ms - an long int containing the progress in ms.
 */
void cb_progress (long int progress_ms)
{
	progress_ms = progress_ms;
}

/**
 * Listens for CTRL-C and does it's best to free resources in a Good Way
 */
static void sigproc(int signum)
{
	signum = signum;
	(void) signal(SIGINT, sigproc);	/* reset the signal */
	endwin ();
	printw ("Recieved ctrl-c. Cleaning up and exiting.\n");
	daisy_term (NULL);
	exit (0);
}
